#!/usr/bin/env node
var _a;
// import necessary files
import { writeFileSync, readdirSync, readFileSync, existsSync, copyFileSync, mkdirSync, renameSync, chmodSync, } from 'fs';
import * as os from 'os';
// import meow from 'meow'
import * as rmdir from 'rimraf';
import * as forge from 'node-forge';
import keypair from 'keypair';
import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
// Get path for work
export var HOME_PATH = "".concat(os.homedir(), "/");
export var SSH_PATH = "".concat(HOME_PATH, ".ssh/");
export var SWITCH_FILE = "".concat(SSH_PATH, ".sshwitch");
// Get the all directory aka keys
export var getDirectories = function (source) {
    var _a, _b;
    return (_b = (_a = readdirSync(source, {
        withFileTypes: true,
    })) === null || _a === void 0 ? void 0 : _a.filter(function (dirent) { return dirent.isDirectory(); })) === null || _b === void 0 ? void 0 : _b.map(function (dirent) { return dirent.name; });
};
// Get current key
export var getCurrent = function () {
    var _a;
    try {
        return readFileSync(SWITCH_FILE).toString();
    }
    catch (e) {
        writeFileSync(SWITCH_FILE, '');
        return (_a = readFileSync(SWITCH_FILE)) === null || _a === void 0 ? void 0 : _a.toString();
    }
};
// get all ssh keys
export var getList = function () {
    var _a;
    (_a = getDirectories(SSH_PATH)) === null || _a === void 0 ? void 0 : _a.map(function (item) { return console.log(item); });
    return getDirectories(SSH_PATH);
};
// Switch ssh key
export var switchKey = function (name) {
    try {
        if (existsSync("".concat(SSH_PATH + name, "/id_rsa")) &&
            existsSync("".concat(SSH_PATH + name, "/id_rsa.pub"))) {
            try {
                copyFileSync("".concat(SSH_PATH + name, "/id_rsa"), "".concat(SSH_PATH, "id_rsa"));
                copyFileSync("".concat(SSH_PATH + name, "/id_rsa.pub"), "".concat(SSH_PATH, "id_rsa.pub"));
                writeFileSync(SWITCH_FILE, name);
                chmodSync("".concat(SSH_PATH, "/id_rsa.pub"), 384);
                chmodSync("".concat(SSH_PATH, "/id_rsa"), 384);
                return console.log("Changed key pair to: ".concat(name));
            }
            catch (err) {
                console.log("Could not copy, check if you have permission to write on ".concat(SSH_PATH));
                return console.error(err);
            }
        }
        else {
            console.log("Could not read key pair in ".concat(SSH_PATH + name));
            console.log("Check if key pair exists in ".concat(SSH_PATH + name, "\n\n"));
            console.log("If not you can create a new one with: \"sshmulti -n ".concat(name, "\""));
            return console.log("Or backup the current key pair in ".concat(SSH_PATH, " with: \"sshmulti -b ").concat(name, "\""));
        }
    }
    catch (err) {
        return console.error(err);
    }
};
// Register new ssh key
export var newKey = function (name) {
    if (existsSync(SSH_PATH + name)) {
        return console.log("".concat(SSH_PATH + name, " already exists, skipping"));
    }
    console.log("Creating key pair in ".concat(SSH_PATH + name));
    try {
        mkdirSync(SSH_PATH + name);
        var pair = keypair();
        var publicKey = forge.pki.publicKeyFromPem(pair.public);
        var privateKey = forge.pki.privateKeyFromPem(pair.private);
        var ssh = forge.ssh.publicKeyToOpenSSH(publicKey);
        var sshPrivate = forge.ssh.privateKeyToOpenSSH(privateKey);
        writeFileSync("".concat(SSH_PATH + name, "/id_rsa.pub"), ssh);
        writeFileSync("".concat(SSH_PATH + name, "/id_rsa"), sshPrivate);
        chmodSync("".concat(SSH_PATH + name, "/id_rsa.pub"), 384);
        chmodSync("".concat(SSH_PATH + name, "/id_rsa"), 384);
        return console.log("\n public key: \n\n ".concat(ssh, "\n private key: \n\n ").concat(sshPrivate));
    }
    catch (err) {
        console.log("Could not create key pair, check if you have permission to write on ".concat(SSH_PATH + name));
        return console.log(err);
    }
};
// backup ssh keys
export var backupKey = function (name) {
    if (existsSync(SSH_PATH + name)) {
        return console.log("".concat(SSH_PATH + name, " already exists, skipping"));
    }
    console.log("Copying current key pair in ".concat(SSH_PATH, " to ").concat(SSH_PATH + name));
    try {
        mkdirSync(SSH_PATH + name);
        copyFileSync("".concat(SSH_PATH, "id_rsa"), "".concat(SSH_PATH + name, "/id_rsa"));
        copyFileSync("".concat(SSH_PATH, "id_rsa.pub"), "".concat(SSH_PATH + name, "/id_rsa.pub"));
        return console.log("Backup to current key pair in ".concat(SSH_PATH, " to ").concat(SSH_PATH + name));
    }
    catch (err) {
        console.log("Could not backup key pair, check if you have permission to write on ".concat(SSH_PATH + name));
        return console.log(err);
    }
};
// Rename ssh key
export var renameKey = function (oldName, newName) {
    if (!existsSync(SSH_PATH + oldName)) {
        return console.log("".concat(SSH_PATH + oldName, " doesn't exists, cannot rename"));
    }
    if (existsSync(SSH_PATH + newName)) {
        return console.log("".concat(SSH_PATH + newName, " already exists, cannot rename"));
    }
    try {
        renameSync(SSH_PATH + oldName, SSH_PATH + newName);
        if (getCurrent() === oldName) {
            writeFileSync(SWITCH_FILE, newName);
            switchKey(newName);
        }
        return console.log('Renamed the current key');
    }
    catch (err) {
        console.log('Could not rename');
        return console.log(err);
    }
};
// delete ssh key
export var deleteKey = function (name) {
    try {
        rmdir.sync(SSH_PATH + name);
        return console.log("Deleted ".concat(name, " sshkey."));
    }
    catch (e) {
        return console.log(e);
    }
};
// Using argv for taking flag and input
var argv = yargs(hideBin(process.argv))
    .usage("\n    Switch and manage key pairs in ".concat(SSH_PATH, "\n    Default usage: sshmulti key_pair_name\n    Advanced usage: sshmulti [option]\n    -c, --current                    Get name of current key pair\n    -n, --new                        New key pair name\n    -b, --backup                     Backup key pair in ").concat(SSH_PATH, "\n    -r, --rename                     Rename a key pair\n    -l, --list                       Get list of key pairs in ").concat(SSH_PATH, "\n    -d, --delete DELETE              Delete key pair\n    "))
    .help('help')
    .alias('help', 'h')
    .version('version', '3.0.1')
    .alias('version', 'V')
    .options({
    backup: { type: 'boolean', alias: ['backup', 'b'], default: false },
    new: { type: 'boolean', alias: ['new', 'n'], default: false },
    rename: { type: 'boolean', alias: ['rename', 'r'], default: false },
    list: { type: 'boolean', alias: ['list', 'l'], default: false },
    current: { type: 'boolean', alias: ['current', 'c'], default: false },
    delete: { type: 'boolean', alias: ['delete', 'd'], default: false },
})
    .parseSync();
// Let the fun began
var isBackup = argv.backup;
var isCurrent = argv.current;
var isDelete = argv.delete;
var isList = argv.list;
var isRename = argv.rename;
var isNew = argv.new;
var name = argv._ || '';
switch (true) {
    case isBackup && ((_a = name[0].toString()) === null || _a === void 0 ? void 0 : _a.length) === 1:
        backupKey(name[0].toString());
        break;
    case isRename && name.length === 2:
        // eslint-disable-next-line no-case-declarations
        if (name[0] && name[1]) {
            renameKey(name[0].toString(), name[1].toString());
        }
        else {
            console.log('Please enter valid name. Type sshmulti --help for more.');
        }
        break;
    case isNew && name.length === 1:
        newKey(name[0].toString());
        break;
    case isDelete && name.length === 1:
        deleteKey(name[0].toString());
        break;
    case isList:
        getList();
        break;
    case isCurrent || name.length === 0:
        console.log(getCurrent());
        break;
    case !isBackup &&
        !isCurrent &&
        !isDelete &&
        !isList &&
        !isRename &&
        !isNew &&
        name.length === 1:
        switchKey(name[0].toString());
        break;
    case name.length > 1:
        console.log('Enter sshmulti --help for more command');
        break;
    default:
        console.log(getCurrent());
        break;
}
export default getDirectories;
